'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = undefined;

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }(); /**
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      * Created by rom on 12/01/2016.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      */

var _chance = require('chance');

var _chance2 = _interopRequireDefault(_chance);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _bluebird = require('bluebird');

var _bluebird2 = _interopRequireDefault(_bluebird);

var _connection = require('./connection');

var _connection2 = _interopRequireDefault(_connection);

var _listener = require('./listener');

var _listener2 = _interopRequireDefault(_listener);

var _sender = require('./sender');

var _sender2 = _interopRequireDefault(_sender);

var _logger = require('./logger');

var _logger2 = _interopRequireDefault(_logger);

var _util = require('./util');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var chance = new _chance2.default();

/**
 * Worker client
 *
 * How does it work?
 * The client connect to RabbitMQ, it send the message to the microservice queue / exchange,
 * and listen to dedicated queue for replies.
 * Both of the replies, the legal and the errors replies received in this queue.
 *
 * When new job is sent with the client, it is send with a new random generated GUID
 * and new Promise is created. The reject and the resolve functions of this Promise are saved in an
 * object in the Client instance (defined by the GUID), and when new message received in the reply
 * or the error queue, it received with the GUID, and the Promise resolve or rejected with the
 * message.
 */

var Client = function () {
  /**
   * Creates a new client
   * @param domain The domain name, will be added to topic names when working with this client.
   * @param asyncResponseQueue A queue to send async responses to
   * @param consumerTag Consumer tag that identifies the client
   * @param rabbitUrl
   * @param defaultTimeout
   */
  function Client(_ref) {
    var domain = _ref.domain,
        asyncResponseQueue = _ref.asyncResponseQueue,
        consumerTag = _ref.consumerTag,
        _ref$rabbitUrl = _ref.rabbitUrl,
        rabbitUrl = _ref$rabbitUrl === undefined ? undefined : _ref$rabbitUrl,
        _ref$defaultTimeout = _ref.defaultTimeout,
        defaultTimeout = _ref$defaultTimeout === undefined ? 30000 : _ref$defaultTimeout;

    _classCallCheck(this, Client);

    // RabbitMQ connection
    rabbitUrl = rabbitUrl || process.env.RABBIT_URL;
    this.conn = new _connection2.default(rabbitUrl);
    // Task list, save the resolve and the reject functions for each task
    this.tasks = {};
    // Define queue names
    this.queueName = undefined;
    // Set Async Response Queue
    this.asyncResponseQueue = asyncResponseQueue;
    // Optional - specify consumer tag
    this.consumerTag = consumerTag ? consumerTag + '-client' : undefined;
    // Set domain name
    this.domain = domain;
    this.defaultTimeout = defaultTimeout;
    if (_lodash2.default.isNil(domain)) {
      throw new Error('domain is mandatory');
    }
    // Pre Hooks
    this.preHook = function clientPreHook(domain, server, topic, msg, meta, context, replyOpt) {}; // eslint-disable-line
  }

  /**
   * Connect to RabbitMQ
   * @returns {Promise}
   */


  _createClass(Client, [{
    key: 'connect',
    value: function connect() {
      var _this = this;

      return new _bluebird2.default(function (resolve) {
        _this.conn.connect();
        _this.conn.on('open', function () {
          _this.configureQueues().then(function () {
            return _this.configureComm();
          }).then(function () {
            return _this.start();
          }).then(resolve);
        });
      });
    }

    /**
     * Assert the queue and save the name in this.queueName
     * @returns {*}
     */

  }, {
    key: 'configureQueues',
    value: function configureQueues() {
      var _this2 = this;

      // Create the replies queue
      return this.conn.channel.assertQueue(this.queueName, { exclusive: true }).then(function (queue) {
        // Save queue name
        _this2.queueName = queue.queue;
        return true;
      });
    }

    /**
     * Configure the listener and the sender
     * @returns {*}
     */

  }, {
    key: 'configureComm',
    value: function configureComm() {
      // Configure listener
      this.listener_reply = new _listener2.default(this.conn.channel, this.queueName, this.consumerTag);

      // Configure sender
      this.sender = new _sender2.default(this.conn.channel);

      return _bluebird2.default.resolve();
    }

    /**
     * Start the listener, and set the logic when a new message received.
     * @returns {*}
     */

  }, {
    key: 'start',
    value: function start() {
      var _this3 = this;

      // Configure listeners events for reply queue
      this.listener_reply.on('msg', function (msg) {
        _this3.sender.ack(msg);
        var msgId = msg.properties.correlationId;
        if (msgId) {
          var isError = !!msg.properties.headers.isError;
          if (isError) {
            // reject with a string instead of an error,
            // because we parse the string into an error in rabbitService.js
            _this3.tasks[msgId].reject(msg);
          } else {
            _this3.tasks[msgId].resolve(msg);
          }
          delete _this3.tasks[msgId];
        }
      });

      // Start listener
      this.listener_reply.start();

      return _bluebird2.default.resolve();
    }

    /**
     * Close the connection to RabbitMQ and close the runner
     */

  }, {
    key: 'close',
    value: function close() {
      var _this4 = this;

      _logger2.default.instance.info('[AMQP] Stopping client...');
      return this.listener_reply.stop().then(function () {
        return _this4.conn.close();
      });
    }

    /**
     * Send new message to the worker
     * Return new Promise, and save the resolve and the reject in an object
     * The listener events will resolve or object this Promise by the id
     * @param server
     * @param topic
     * @param msg - JSON, string, array, or whatever
     * @param meta
     * @param timeout - for sync tasks only
     * @param context
     * @param replyOpt
     * @returns {Promise}
     * @private
     */

  }, {
    key: '_send',
    value: function _send(server, topic, msg) {
      var meta = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : {};
      var timeout = arguments[4];

      var _this5 = this;

      var context = arguments.length > 5 && arguments[5] !== undefined ? arguments[5] : {};
      var replyOpt = arguments.length > 6 && arguments[6] !== undefined ? arguments[6] : {};

      this.preHook(this.domain, server, topic, msg, meta, context, replyOpt);

      var topicWithDomain = (0, _util.buildTopic)(this.domain, topic);

      // Generate Task ID
      var id = chance.guid();

      // Set MicroService exchange and queue names
      if (!server || !server.serverExchangeName && !server.serverQueueName) {
        throw new Error('You have to set serverExchangeName or serverQueueName');
      }

      // Convert message to Buffer
      msg = new Buffer(JSON.stringify(msg));

      // Set replyTo and replyToTopic
      var replyTo = _lodash2.default.get(replyOpt, 'queue', this.queueName);
      var replyToTopic = _lodash2.default.get(replyOpt, 'topic');
      timeout = timeout === Number.MAX_SAFE_INTEGER ? undefined : timeout || this.defaultTimeout;
      // Create a promise for the response
      var responsePromise = new _bluebird2.default(function (resolve, reject) {
        // Append task to task list only if the task supposed to return to the client (sync task)
        if (replyTo === _this5.queueName) {
          _this5.tasks[id] = { resolve: resolve, reject: reject };
        }
      }).timeout(timeout);

      // Send the message
      var sendPromise = new _bluebird2.default(function (resolve, reject) {
        var options = {
          replyTo: replyTo,
          headers: { domain: _this5.domain, topic: topic, replyToTopic: replyToTopic, context: context, meta: meta },
          messageId: id
        };
        if (timeout) options.expiration = timeout;

        var publish = _this5.sender.publish(server.serverExchangeName, server.serverQueueName || topicWithDomain, msg, options);
        if (publish) resolve();else reject(new Error('publish error'));
      });

      // Return both of the promises
      return { responsePromise: responsePromise, sendPromise: sendPromise };
    }

    /**
     * Send and wait to answer, return a Promise with the answer
     * @param server
     * @param topic
     * @param msg
     * @param meta
     * @param timeout
     * @returns {Promise}
     */

  }, {
    key: 'sendSync',
    value: function sendSync(server, topic, msg, meta, timeout) {
      var send = this._send(server, topic, msg, meta, timeout, {}, {});
      return send.responsePromise;
    }

    /**
     * Send and forget from the answer, the asyncResponseQueue will handle the response
     * @param server
     * @param topic
     * @param msg
     * @param meta
     * @param context
     * @param replyOpt
     * @returns {Promise}
     */

  }, {
    key: 'sendAsync',
    value: function sendAsync(server, topic, msg, meta, context, replyOpt) {
      var send = this._send(server, topic, msg, meta, Number.MAX_SAFE_INTEGER, context, _lodash2.default.extend({}, replyOpt, { queue: this.asyncResponseQueue }));
      return send.sendPromise;
    }
  }]);

  return Client;
}();

exports.default = Client;