'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }(); /**
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      * Created by chenrozenes and ofirhartuv on 05/11/2017.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      */


var _bluebird = require('bluebird');

var _bluebird2 = _interopRequireDefault(_bluebird);

var _logger = require('../logger');

var _logger2 = _interopRequireDefault(_logger);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var DEFAULT_INTERVAL = 1000;

/**
 * Base class that performs the pull logic from a queue
 * This class won't sync between different workers, and needs to be inherited in order to do so
 * (checkout PostgresPuller)
 */

var Puller = function () {
  function Puller() {
    var config = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    _classCallCheck(this, Puller);

    this.interval = config.interval || DEFAULT_INTERVAL;
    this.isRunning = false;
  }

  /**
   * Returns true if current worker can pull from queue
   */


  _createClass(Puller, [{
    key: 'shouldPull',
    value: function shouldPull() {
      return _bluebird2.default.resolve(true);
    }

    /**
     * Pulls a message from a queue and calls the callback only pulled something
     * @param channel
     * @param queue
     * @param callback
     * @returns {*}
     */

  }, {
    key: 'pull',
    value: function pull(channel, queue, callback) {
      return channel.get(queue, { noAck: false }).then(function (msg) {
        // false means that there's no messages to pull in the queue
        if (msg !== false) {
          callback(msg);
        }
      });
    }

    /**
     * Pulling loop -
     * Checks if pulling is possible, pulls the message and tries again after waiting the interval
     * @param channel
     * @param queue
     * @param callback
     * @returns {Promise.<T>|*}
     */

  }, {
    key: 'tryPull',
    value: function tryPull(channel, queue, callback) {
      var _this = this;

      if (!this.isRunning) return _bluebird2.default.resolve();

      return this.shouldPull().then(function (shouldPull) {
        if (shouldPull) return _this.pull(channel, queue, callback);
        return _bluebird2.default.resolve();
      }).then(function () {
        return _bluebird2.default.delay(_this.interval);
      }).then(function () {
        _this.tryPull(channel, queue, callback);
      }).catch(function (error) {
        _logger2.default.instance.error('[AMQP] Error occurred when trying to pull. ' + 'continuing on the next interval', { error: error });

        _this.tryPull(channel, queue, callback);
      });
    }

    /**
     * Starts Pulling loop
     * @param channel
     * @param queue
     * @param callback
     * @returns {Promise.<T>|*}
     */

  }, {
    key: 'start',
    value: function start(channel, queue, callback) {
      this.channel = channel;
      this.queue = queue;

      this.isRunning = true;
      _logger2.default.instance.debug('[AMQP] Start pulling from queue ' + queue);
      return this.tryPull(channel, queue, callback);
    }

    /**
     * Stops the pulling loop
     */

  }, {
    key: 'stop',
    value: function stop() {
      _logger2.default.instance.debug('[AMQP] Stopping pulling');
      this.isRunning = false;
      // Wait for loop to stop
      return _bluebird2.default.delay(this.interval * 2);
    }
  }]);

  return Puller;
}();

exports.default = Puller;