'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = undefined;

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }(); /**
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      * Created by asafdavid on 10/12/2015.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      */


var _bluebird = require('bluebird');

var _bluebird2 = _interopRequireDefault(_bluebird);

var _logger = require('./logger');

var _logger2 = _interopRequireDefault(_logger);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var DEFAULT_MAX_ATTEMPTS = 3;
var DEFAULT_MESSAGE_TIMEOUT_INTERVAL = 5000;
var DEFAULT_QUEUE_EXPIRATION = 90000;

/**
 * Responsible for sending messages to the AMQP server
 */

var Sender = function () {
  /**
   * Creates a new sender
   * @param channel
   * @param errorExchange
   * @param nackExchange
   * @param timeoutExchange
   * @param timeoutQueueNamePrefix
   * @param queueExpiration
   */
  function Sender(channel, errorExchange, nackExchange, timeoutExchange, timeoutQueueNamePrefix, queueExpiration) {
    _classCallCheck(this, Sender);

    this.channel = channel;
    this.errorExchange = errorExchange;
    this.nackExchange = nackExchange;
    this.timeoutExchange = timeoutExchange;
    this.timeoutQueueNamePrefix = timeoutQueueNamePrefix;
    this.queueExpiration = queueExpiration;
  }

  /**
   * Publishes a message using through the opened channel
   * @param exchange
   * @param queue
   * @param content
   * @param options
   */


  _createClass(Sender, [{
    key: 'publish',
    value: function publish(exchange, routingKey, content, options) {
      return this.channel.publish(exchange, routingKey, content, options);
    }

    /**
     * Publishes a message directly to the provided queue
     * @param exchange
     * @param queue
     * @param content
     * @param options
     */

  }, {
    key: 'sendToQueue',
    value: function sendToQueue(queue, content, options) {
      var _this = this;

      // sendToQueue doesn't have a promise version
      return new _bluebird2.default(function (resolve, reject) {
        _this.channel.sendToQueue(queue, content, options, function (err, res) {
          if (err) {
            reject(err);
          } else {
            resolve(res);
          }
        });
      });
    }

    /**
     * Extracts an option object based on rabbit-task message format
     * @param {Message} msg
     * @param {Object} options
     * @returns {{}}
     */

  }, {
    key: 'error',


    /**
     * Reports an error to the error queue
     * @param msg
     * @param reason
     */
    value: function error(msg, reason) {
      _logger2.default.instance.error('[Sender] reports an error of a msg', { consumerTag: msg.fields.consumerTag });
      return this.publish(this.errorExchange, msg.fields.routingKey, msg.content, {
        headers: {
          error: reason
        }
      });
    }

    /**
     * Applicative nack,
     * verifies if the maximum attempts was reached,
     * in case not publishes the message to the nack exchange.
     * the message would be acked anyway.
     * @param msg
     */

  }, {
    key: 'appNack',
    value: function appNack(msg) {
      var _this2 = this;

      var attempts = msg.headers['x-attempts-left'];
      attempts = attempts ? attempts - 1 : DEFAULT_MAX_ATTEMPTS - 1;

      var timeoutInterval = msg.headers['time-out-interval'] ? msg.headers['time-out-interval'] : DEFAULT_MESSAGE_TIMEOUT_INTERVAL;
      if (attempts > 0) {
        // message ttl is important because of 2 reasons:
        // 1) to make sure we can track the the message timeout
        // 2) for the routing in the timeout exchange.
        var options = Sender.buildOptionsFromMsg(msg, {
          'x-attempts-left': attempts,
          'message-ttl': timeoutInterval
        });
        options.replyTo = msg.replyTo;
        // make sure we have the correct queue and bind it to the EXCHANGE
        var queueName = this.timeoutQueueNamePrefix + '.timeout.' + timeoutInterval;
        this.channel.assertQueue(queueName, {
          deadLetterExchange: this.nackExchange,
          arguments: {
            'x-message-ttl': timeoutInterval,
            'x-expires': this.queueExpiration || DEFAULT_QUEUE_EXPIRATION
          }
        }).then(function () {
          return (
            // we do the routing on this exchange by header. so in this case all messages with the
            // same timeout will be routed to the same queue.
            _this2.channel.bindQueue(queueName, _this2.timeoutExchange, '', {
              'x-match': 'any',
              'message-ttl': timeoutInterval
            })
          );
        }).then(function () {
          return _this2.publish(_this2.timeoutExchange, msg.fields.routingKey, msg.content, options);
        });
      }
      return this.ack(msg.raw);
    }

    /**
     * Sends a response to a queue
     * @param {HandlerResponse} response
     * @returns {*}
     */

  }, {
    key: 'reply',
    value: function reply(response) {
      if (response.replyTo) {
        // create a buffer from body if exists. otherwise an empty buffer
        return this.sendToQueue(response.replyTo, new Buffer(response.body || 0), response.toRabbitOptions());
      }

      return this.error(response.msg, 'No reply queue was defined').then(function () {
        return _bluebird2.default.reject(new Error('No reply queue was defined'));
      });
    }

    /**
     * Acks a message
     * @param msg
     */

  }, {
    key: 'ack',
    value: function ack(msg) {
      return this.channel.ack(msg);
    }

    /**
     * Nacks a message
     * @param msg
     * @returns {{deliveryTag, multiple, requeue}}
     */

  }, {
    key: 'nack',
    value: function nack(msg) {
      return this.channel.nack(msg);
    }
  }], [{
    key: 'buildOptionsFromMsg',
    value: function buildOptionsFromMsg(msg, options) {
      return msg.toRabbitOptions(options);
    }
  }]);

  return Sender;
}();

exports.default = Sender;