'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = Task;

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _baseService = require('../baseService');

var _baseService2 = _interopRequireDefault(_baseService);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * A method decorator to define a new task in a micro-service interface,
 * the decorator will add the right metadata to the
 * @param sync
 * @param description
 * @param topic
 * @param errors
 * @param baseError
 * @param timeout
 * @returns {taskDecorator}
 */
/**
 * Created by asafdavid on 20/02/2017.
 */
function Task(_ref) {
  var sync = _ref.sync,
      description = _ref.description,
      topic = _ref.topic,
      _ref$errors = _ref.errors,
      errors = _ref$errors === undefined ? [] : _ref$errors,
      baseError = _ref.baseError,
      timeout = _ref.timeout;

  if (sync && !_lodash2.default.isBoolean(sync)) throw new TypeError('sync must be a boolean');
  if (description && !_lodash2.default.isString(description)) throw new TypeError('description must be a string');

  return function taskDecorator(target, key, descriptor) {
    // Check if a task is defined within a service
    if (!(target instanceof _baseService2.default)) {
      throw SyntaxError('@task must be defined within a service');
    }
    if (!_lodash2.default.isFunction(descriptor.value)) throw SyntaxError('@task must be defined on a function');

    // Initializes the _tasks container
    if (!target._tasks) target._tasks = {};

    // Define the task metadata
    var task = {
      name: key,
      handler: descriptor.value,
      middleware: { pre: [], post: [] },
      options: {},
      sync: sync,
      topic: topic,
      description: description,
      errors: errors,
      baseError: baseError,
      timeout: timeout
    };

    // merge with father class
    var father = Object.getPrototypeOf(target);
    var fatherTask = father instanceof _baseService2.default ? father._tasks[key] : undefined;
    target._tasks[key] = _lodash2.default.merge({}, fatherTask, task);

    var topicInTask = target._tasks[key].topic;
    if (!topicInTask || !_lodash2.default.isString(topicInTask)) throw new TypeError('topic is required and must be a string');
  };
}